import { Subscription } from 'rxjs';

import { Component, ViewChild, OnDestroy, OnInit, Type } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';

import { MatTabChangeEvent } from '@angular/material/tabs';
import { MatSidenav } from '@angular/material/sidenav';

import { Store, select } from '@ngrx/store';

import { LogService } from '@ucap/ng-logger';

import { AppSelector } from '@app/store/state';

@Component({
  selector: 'app-layouts-default',
  templateUrl: './default.layout.component.html',
  styleUrls: ['./default.layout.component.scss']
})
export class DefaultLayoutComponent implements OnInit, OnDestroy {
  @ViewChild('leftSidenav', { static: true })
  leftSidenav: MatSidenav;

  leftSectionComponent: Type<any>;
  contentSectionComponent: Type<any>;

  private windowSizeSubscription: Subscription;
  private routerEventSubscription: Subscription;

  constructor(
    private router: Router,
    private activatedRoute: ActivatedRoute,
    private store: Store<any>,
    private logService: LogService
  ) {
    this.routerEventSubscription = this.router.events.subscribe((event) => {
      this.logService.debug('DefaultLayoutComponent', event);
    });
  }

  ngOnInit(): void {
    this.windowSizeSubscription = this.store
      .pipe(select(AppSelector.windowSize))
      .subscribe((size) => {
        if (size.width < 780) {
          if (this.leftSidenav.opened) {
            this.leftSidenav.close();
          }
        } else {
          if (!this.leftSidenav.opened) {
            this.leftSidenav.open();
          }
        }
      });
  }

  ngOnDestroy(): void {
    if (!this.windowSizeSubscription) {
      this.windowSizeSubscription.unsubscribe();
    }
    if (!!this.routerEventSubscription) {
      this.routerEventSubscription.unsubscribe();
    }
  }

  onSelectedTabChange(event: MatTabChangeEvent) {
    switch (event.index) {
      case 1:
        this.router.navigate(['/chat']);
        break;
      case 2:
        this.router.navigate(['/organization']);
        break;
      case 3:
        this.router.navigate(['/message']);
        break;
      default:
        this.router.navigate(['/group']);
        break;
    }
  }

  onRoute(url: string) {
    this.logService.debug('onRoute', url);
    switch (url) {
      case '/organization':
        import('@app/sections/organization/organization.section.module').then(
          (m) => {
            this.leftSectionComponent = m.TreeListSectionComponent;
          }
        );
        import('@app/sections/organization/organization.section.module').then(
          (m) => {
            this.contentSectionComponent = m.MembersSectionComponent;
          }
        );
        break;
      case '/group':
        break;
      case '/chat':
        import('@app/pages/chat/chat.page.module').then((m) => {
          this.leftSectionComponent = m.RoomListPageComponent;
          this.contentSectionComponent = m.IndexPageComponent;
        });
        break;
      case '/message':
        break;

      default:
        break;
    }
  }

  onClickToggleLeftSidenav() {
    if (this.leftSidenav.opened) {
      this.leftSidenav.close();
    } else {
      this.leftSidenav.open();
    }
  }
}
